import scipy.io
import numpy as np
from scipy.signal import butter, filtfilt, find_peaks
import matplotlib.pyplot as plt

def load_emg_data(file_path):

    mat_data = scipy.io.loadmat(file_path)
    
    if 'emg_data_walking' not in mat_data:
        raise KeyError("De verwachte variabele 'emg_data_walking' is niet in het bestand aanwezig.")
    
    emg_data = mat_data['emg_data_walking']
    
    channel_names = emg_data['data_headers'][0][0]
    channel_names = [ch[0] for ch in channel_names[0]]
    

    print("EMG-kanalen:")
    for idx, name in enumerate(channel_names, start=1):
        print(f"{idx-1}. {name}")
    


    return channel_names


file_path = "emg_data_walking.mat"
#load_emg_data(file_path)

chosen_muscles = {
    'rectus_fomoris': 0,
    'tibialis_anterior': 7
}
#print(f"Gekozen spieren: {chosen_muscles}")

class DataRetrieval:
    def __init__(self, file_path):
        self.file_path = file_path
        self.emg_data = None
        self.channel_names = None
    
    def load_data(self):
        mat_data = scipy.io.loadmat(self.file_path)
        self.emg_data = mat_data['emg_data_walking']
        self.channel_names = self.emg_data['data_headers'][0][0]
        self.channel_names = [ch[0] for ch in self.channel_names[0]]

    def get_channel_names(self):
        return self.channel_names
    
class SignalProcessing:
    def __init__(self, emg_data, fs):
        self.emg_data = emg_data
        self.fs = fs

    def bandpass_filter(self, data, lowcut, highcut):
        nyquist = 0.5 * self.fs
        low = lowcut / nyquist
        high = highcut / nyquist
        order = 4
        b, a = butter(order, [low, high], btype='band')
        filtered_data = filtfilt(b, a, data)
        return filtered_data
    
    def lowpass_filter(self, data, cutoff):
        nyquist = 0.5 * self.fs
        normal_cutoff = cutoff / nyquist
        order = 4
        b, a = butter(order, normal_cutoff, btype='low')
        filtered_data = filtfilt(b, a, data)
        return filtered_data

    def power_spectral_analysis(self, data):
        N = len(data)
        fft_values = np.fft.fft(data)
        fft_magnitude = np.abs(fft_values[:N//2])
        freq = np.fft.fftfreq(N, d=1/self.fs)[:N//2]
        return freq, fft_magnitude

#print(dir(scipy.signal))


def info_emg_data(file_path):
    mat_data = scipy.io.loadmat(file_path)
    emg_data = mat_data['emg_data_walking']

    sampling_rate = emg_data['sampling_rate'][0][0][0]

    data = emg_data['data'][0][0]
    data = data.T
    num_samples = data.shape[1]
    duration = num_samples / sampling_rate
    return sampling_rate, data, num_samples, duration


file_path = "emg_data_walking.mat"
info_emg_data(file_path)

def compute_fft(signal, sampling_rate):
    N = len(signal)
    fft_values = np.fft.fft(signal)
    fft_magnitude = np.abs(fft_values[:N//2])
    freq = np.fft.fftfreq(N, d=1/sampling_rate)[:N//2]
    return freq, fft_magnitude

def plot_emg_data(file_path):
    sampling_rate, data, num_samples, duration = info_emg_data(file_path)

    sampling_time = float(1/sampling_rate)
    num_samples = int(6 * sampling_rate)
    num_samples = min(num_samples, data.shape[1])
    time_vector = np.linspace(0,6,num_samples)

    
    channel_rectus_femoris = data[chosen_muscles['rectus_fomoris'], :num_samples]
    channel_tibialis_anterior = data[chosen_muscles['tibialis_anterior'], :num_samples]
    
    signal_processor = SignalProcessing(data, sampling_rate)
    filtered_rf = signal_processor.bandpass_filter(channel_rectus_femoris, 25, 300)
    filtered_ta = signal_processor.bandpass_filter(channel_tibialis_anterior, 25, 300)

    rectified_rf = np.abs(filtered_rf)
    rectified_ta = np.abs(filtered_ta)

    envelope_rf = signal_processor.lowpass_filter(rectified_rf,5)
    envelope_ta = signal_processor.lowpass_filter(rectified_ta,5)


    freq_rf, fft_rf = compute_fft(envelope_rf, sampling_rate)
    freq_ta, fft_ta = compute_fft(envelope_ta, sampling_rate)

    peaks_rf, _ = find_peaks(envelope_rf)
    peaks_ta, _ = find_peaks(envelope_ta)

    num_peaks_rf = len(peaks_rf)
    num_peaks_ta = len(peaks_ta)

    max_value_rf = np.max(envelope_rf)
    max_value_ta = np.max(envelope_ta)

    area_rf = np.trapz(envelope_rf, time_vector)
    area_ta = np.trapz(envelope_ta, time_vector)

    print(f"1.Numbers of EMG peaks Rectus Femoris: {num_peaks_rf}")
    print(f"1.Numbers of EMG peaks Tibialis Anterior: {num_peaks_ta}")
    print(f"2. Maximale peak Rectus Femoris: {max_value_rf}")
    print(f"2. Maximale peak Tibialis Anterior: {max_value_ta}")
    print(f"3. Area under Curve Rectus Femoris {area_rf}")
    print(f"3. Area under Curve Tibialis Anterior {area_ta}")

    plt.figure(figsize=(13,8))

    plt.subplot(2, 2, 1)
    plt.plot(time_vector, envelope_rf)
    plt.title('EMG Data - Rectus femoris')
    plt.xlabel('Time [Seconds]')
    plt.ylabel('Amplitude')

    plt.subplot(2, 2, 2)
    plt.plot(time_vector, envelope_ta)
    plt.title('EMG Data - Tibialis Anterior')
    plt.xlabel('Time [seconds]')
    plt.ylabel('Amplitude')

    plt.subplot(2, 2, 3)
    plt.plot(freq_rf, fft_rf)
    plt.title('Power Spectrum - Rectus Femoris')
    plt.xlabel('Frequency [Hz]')
    plt.ylabel('Magnitude')

    plt.subplot(2, 2, 4)
    plt.plot(freq_ta, fft_ta)
    plt.title('Power Spectrum - Tibialis Anterior')
    plt.xlabel('Frequency [Hz]')
    plt.ylabel('Magnitude')

    plt.tight_layout()
    plt.show()

file_path = "emg_data_walking.mat"
plot_emg_data(file_path)




